@@ -2,6 +2,8 @@
 #
 # Template::Plugin::Latex
 #
+# $Id:$
+#
 # DESCRIPTION
 #   Template Toolkit plugin for Latex
 #
@@ -24,9 +26,10 @@
 #   * Removed the program pathname options on the FILTER call
 #     Andrew Ford, 05 June 2006
 #   * Totally rewritten by Andrew Ford September 2007
+#   * Version 3.00 released March 2009
 #
 #========================================================================
- 
+
 package Template::Plugin::Latex;
 
 use strict;
@@ -39,8 +42,8 @@ use LaTeX::Encode;
 use LaTeX::Table;
 
 
-our $VERSION = "3.00_05";
-our $DEBUG   = 0 unless defined $DEBUG;
+our $VERSION = '3.02.01';           # Update "=head1 VERSION" below!!!!
+our $DEBUG; $DEBUG = 0 unless defined $DEBUG;
 our $ERROR   = '';
 our $FILTER  = 'latex';
 our $THROW   = 'latex';        # exception type
@@ -58,25 +61,25 @@ sub new {
 
     # create a closure to generate filters with additional options
     my $filter_factory = sub {
-        my $context = shift;
+        my $factory_context = shift;
         my $filter_opts = ref $_[-1] eq 'HASH' ? pop : { };
         my $filter_args = [ @_ ];
         @$filter_opts{ keys %$options } = values %$options;
         return sub {
             # Template::Plugin::Latex::_filter->run($context, $filter_opts, $filter_args, @_);
-            _tt_latex_filter($class, $context, $filter_opts, $filter_args, @_);
+            _tt_latex_filter($class, $factory_context, $filter_opts, $filter_args, @_);
         };
     };
 
     # create a closure to generate filters with additional options
     my $encode_filter_factory = sub {
-        my $context = shift;
+        my $factory_context = shift;
         my $filter_opts = ref $_[-1] eq 'HASH' ? pop : { };
         my $filter_args = [ @_ ];
         @$filter_opts{ keys %$options } = values %$options;
         return sub {
             # Template::Plugin::Latex::_filter->run($context, $filter_opts, $filter_args, @_);
-            _tt_latex_encode_filter($class, $context, $filter_opts, $filter_args, @_);
+            _tt_latex_encode_filter($class, $factory_context, $filter_opts, $filter_args, @_);
         };
     };
 
@@ -103,7 +106,7 @@ sub new {
 #
 #------------------------------------------------------------------------
 
-sub _tt_latex_encode_filter { 
+sub _tt_latex_encode_filter {
     my ($class, $context, $options, $filter_args, @text) = @_;
     my $text = join('', @text);
     return latex_encode($text, %{$options});
@@ -158,7 +161,7 @@ sub _tt_latex_filter {
                                       texinputs => _setup_texinput_paths($context),
                                     );
         $drv->run;
-    };        
+    };
     if (my $e = LaTeX::Driver::Exception->caught()) {
         $class->_throw("$e");
     }
@@ -227,42 +230,123 @@ Template::Plugin::Latex - Template Toolkit plugin for Latex
 
 =head1 VERSION
 
-This documentation refers to C<Template::Plugin::Latex> version 3.00_03
+This documentation refers to C<Template::Plugin::Latex> version 3.02.01.
 
 =head1 SYNOPSIS
 
-    [%  # sample Template Toolkit code
+Sample Template Toolkit code:
 
-        USE Latex;
+    [%- USE Latex;
 
-        mystr = "a, b & c" FILTER latex_encode;
+        mystr = "a, b & c" | latex_encode;
 
-        mydoc = BLOCK %]
+        FILTER latex("pdf");  -%]
 
     \documentclass{article}
     \begin{document}
-      This is a PDF document generated by 
-      LaTeX and the Template Toolkit, with some 
+      This is a PDF document generated by
+      LaTeX and the Template Toolkit, with some
       interpolated data: [% mystr %]
     \end{document}
 
-    [%  END;
+    [%  END; -%]
 
-        mydoc FILTER latex("pdf");
-
-        # eat all whitespace 
-
-     -%]
 
 =head1 DESCRIPTION
 
-This plugin allows you to use LaTeX to generate PDF, PostScript and
-DVI output files from the Template Toolkit.  
+The C<Latex> Template Toolkit plugin provides a C<latex> filter that
+allows the use of LaTeX to generate PDF, PostScript and DVI output files
+from the Template Toolkit.  The plugin uses L<LaTeX::Driver> to run the
+various LaTeX programs.
+
+Processing of the LaTeX document takes place in a temporary directory
+that is deleted once processing is complete.  The standard LaTeX
+programs (C<latex> or C<pdflatex>, C<bibtex> and C<makeindex>) are run
+and re-run as necessary until all references, indexes, bibliographies,
+table of contents, and lists of figures and tables are stable or it is
+apparent that they will not stabilize.  The format converters C<dvips>,
+C<dvipdf>, C<ps2pdf> and C<pdf2ps> are run as necessary to convert the
+output document to the requested format.  The C<TEXINPUTS> environment
+variable is set up to include the template directory and the C<INCLUDES>
+directories, so that LaTeX file inclusion commands should find the
+intended files.
+
+The output of the filter is binary data (although PDF and PostScript are
+not stictly binary).  You should be careful not to prepend or append any
+extraneous characters (even space characters) or text outside the FILTER
+block as this text will be included in the file output.  Notice in the
+example below how we use the post-chomp flags ('-') at the end of the
+C<USE> and C<END> directives to remove the trailing newline characters:
+
+    [% USE Latex(format='pdf') -%]
+    [% FILTER latex %]
+    ...LaTeX document...
+    [% END -%]
+
+If you're redirecting the output to a file via the third argument of
+the Template module's C<process()> method then you should also pass
+the C<binmode> parameter, set to a true value to indicate that it is a
+binary file.
+
+    use Template;
+
+    my $tt = Template->new({
+        INCLUDE_PATH => '/path/to/templates',
+        OUTPUT_PATH  => '/path/to/pdf/output',
+    });
+    my $vars = {
+        title => 'Hello World',
+    }
+    $tt->process('example.tt2', $vars, 'example.pdf', binmode => 1)
+        || die $tt->error();
+
+If you want to capture the output to a template variable, you can do
+so like this:
+
+    [% output = FILTER latex %]
+    ...LaTeX document...
+    [% END %]
+
+You can pass additional arguments when you invoke the filter, for
+example to specify the output format.
+
+    [% FILTER latex(format='pdf') -%]
+       ...LaTeX document...
+    [% END %]
+
+If you want to write the output to a file then you can specify an
+C<output> parameter.
+
+    [% FILTER latex(output='example.pdf') %]
+    ...LaTeX document...
+    [% END %]
+
+If you don't explicity specify an output format then the filename
+extension (e.g. 'pdf' in the above example) will be used to determine
+the correct format.
+
+You can specify a different filter name using the C<filter> parameter.
+
+    [% USE Latex(filter='pdf') -%]
+    [% FILTER pdf %]
+    ...LaTeX document...
+    [% END %]
+
+You can also specify the default output format.  This value can be
+C<latex>, C<pdf> or C<dvi>.
+
+    [% USE Latex(format='pdf') %]
+
+
+Note: the C<LaTeX::Driver> distribution includes three filter programs
+(C<latex2dvi>, C<latex2pdf> and C<latex2ps>) that use the
+C<LaTeX::Driver> package to process LaTeX source data into DVI, PDF or
+PostScript file respectively.  These programs have a C<-tt2> option to
+run their input through the Template Toolkit before processing as LaTeX
+source.  The programs do not use the C<Latex> plugin unless the template
+requests it, but they may provide an alternative way of processing
+Template Toolkit templates to generate typeset output.
 
-The C<latex> filter was distributed as part of the core Template
-Toolkit until version 2.15 released in May 2006 when it was moved into
-the separate Template-Latex distribution.  It should now be loaded as
-a plugin to enable the C<latex> filter:
 
 =head1 SUBROUTINES/METHODS
 
@@ -338,13 +422,13 @@ C<^> (superscript), C<~> (non-breakable space), C<$> (mathematics mode).
 Lists the characters that should be excluded from encoding.  By
 default no special characters are excluded, but it may be useful to
 specify C<except = "\\{}"> to allow the input string to contain LaTeX
-commands such as C<"this is \\textbf{bold} text">.
+commands such as C<"this is \textbf{bold} text">.
 
 =item C<use_textcomp>
 
 By default the C<latex_encode> filter will encode characters with the
 encodings provided by the C<textcomp> LaTeX package (for example the
-Pounds Sterling symbol is encoded as C<\\textsterling{}>).  Setting
+Pounds Sterling symbol is encoded as C<\textsterling{}>).  Setting
 C<use_textcomp = 0> turns off these encodings.
 
 =back
@@ -367,116 +451,67 @@ The following example shows how a simple table can be set up.
                             headings = [ [ 'City', 'Country' ] ],
                             data     = data );
      -%]
-                            
+
 The variable C<text> will hold the LaTeX commands to typeset the table
 and can be further interpolated into a LaTeX document template.
 
-TO BE EXPANDED.
+=head1 DIAGNOSTICS
 
+Most failures result from invalid LaTeX input and are propogated up from
+L<LaTeX::Driver>, L<LaTeX::Encode> or L<LaTeX::Table>.
 
-=head1 OLD DOCUMENTATION
+Failures detected in this module include:
 
+=over 4
 
-    [% USE Latex -%]
-    [% FILTER latex %]
-    ...LaTeX document...
-    [% END %] 
+=item C<OUTPUT_PATH is not set>
 
-You can specify a different filter name using the C<filter> parameter.
-
-    [% USE Latex(filter='pdf') -%]
-    [% FILTER pdf %]
-    ...LaTeX document...
-    [% END %] 
-
-You can also specify the default output format.  This value can be
-C<latex>, C<pdf> or C<dvi>.
-
-    [% USE Latex(format='pdf') %]
-
-With the plugin loaded and a default format defined, you can now use
-the C<latex> filter.
-
-    [% FILTER latex -%]
-    \documentclass{article}
-    \begin{document}
-    This is a PDF document generated by 
-    Latex and the Template Toolkit.
-    \end{document}
-    [% END %]
-
-You can pass additional arguments when you invoke the filter to 
-specify the output format.
-
-    [% FILTER latex(format='pdf') -%]
-       ...LaTeX document...
-    [% END %]
-
-The template content between the C<FILTER> and C<END> directives will
-be piped to the appropriate program(s) to generate the document
-output.  This is fine if you're generating a document directly from a
-template.  For example:
-
-F<example.pdf>:
-
-    [% USE Latex(format='pdf') -%]
-    [% FILTER latex %]
-    ...LaTeX document...
-    [% END -%]
-
-The output will be a binary format PDF, PostScript or DVI file.  You
-should be careful not to prepend or append any extraneous characters
-or text outside the FILTER block as this text will be included in the
-file output.  Notice in the above example how we use the post-chomp
-flags ('-') at the end of the C<USE> and C<END> directives to remove
-the trailing newline characters.
-
-If you're redirecting the output to a file via the third argument of
-the Template module's C<process()> method then you should also pass
-the C<binmode> parameter set to a true value to indicate that it is a
-binary file.
-
-    use Template;
-    
-    my $tt = Template->new({
-        INCLUDE_PATH => '/path/to/templates',
-        OUTPUT_PATH  => '/path/to/pdf/output',
-    });
-    my $vars = {
-        title => 'Hello World',
-    }
-    $tt->process('example.tt2', $vars, 'example.pdf', binmode => 1)
-        || die $tt->error();
-
-If you want to capture the output to a template variable, you can do
-so like this:
-
-    [% output = FILTER latex %]
-    ...LaTeX document...
-    [% END %]
-
-If you want to write the output to a file then you can specify an 
-C<output> parameter.
-
-    [% FILTER latex(output='example.pdf') %]
-    ...LaTeX document...
-    [% END %]
-
-If you don't explicity specify an output format then the filename
-extension (e.g. 'pdf' in the above example) will be used to determine
-the correct format.
+an output filename was specified but the C<OUTPUT_PATH> configuration
+option has not been set.
 
+=back
 
 
 =head1 DEPENDENCIES
 
+=over 4
+
+=item L<Template>
+
+The Template Toolkit.
+
+=item L<LaTeX::Driver>
+
+Provides the logic for driving the LaTeX programs.
+
+=item L<LaTeX::Encode>
+
+Underpins the C<latex_encode> filter.
+
+=item L<LaTeX::Table>
+
+Underpins the C<table> function.
+
+=back
+
+
+=head1 INCOMPATIBILITIES
+
+The C<latex> filter was distributed as part of the core
+Template Toolkit distribution until version 2.15 (released in May 2006),
+when it was moved into the separate Template-Latex distribution.  The
+C<Latex> plugin must now be explicity to enable the C<latex> filter.
+
+
 =head1 BUGS AND LIMITATIONS
 
 The paths to the F<latex>, F<pdflatex> and F<dvips> should be
-pre-defined as part of the installation process (i.e. when you run
-C<perl Makefile.PL>).  You can specify alternate values as
-configuration options to the C<Template> constructor in the Perl
-inteface, but there are no option to specify these paths in template code as this 
+pre-defined as part of the installation process of L<LaTeX::Driver>
+(i.e. when you run C<perl Makefile.PL> for that package).  Alternative
+values can be specified from Perl code using the C<program_path> class
+method from that package, but there are deliberately no options to
+specify these paths from TT code.
+
 
 =head1 AUTHOR
 
@@ -485,11 +520,12 @@ Andrew Ford E<lt>a.ford@ford-mason.co.ukE<gt> (current maintainer)
 Andy Wardley E<lt>abw@wardley.orgE<gt> L<http://wardley.org/>
 
 The original Latex plugin on which this is based was written by Craig
-Barratt with additions for Win32 by Richard Tietjen.
+Barratt with additions for Win32 by Richard Tietjen.  The code has
+subsequently been radically refactored by Andrew Ford.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (C) 2006-2007 Andrew Ford.  All Rights Reserved.
+Copyright (C) 2006-2009 Andrew Ford.  All Rights Reserved.
 
 Copyright (C) 1996-2006 Andy Wardley.  All Rights Reserved.
 
@@ -502,13 +538,15 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 =head1 SEE ALSO
 
-L<Template::Latex>, L<LaTeX::Driver>, L<LaTeX::Table>
+L<Template>, L<LaTeX::Driver>, L<LaTeX::Table>, L<LaTeX::Encode>
+
+
+L<latex2dvi(1)>, L<latex2pdf(1)> and L<latex2ps(1)> (part of the
+C<LaTeX::Driver> distribution)
 
 =cut
 
 
-
-
 # Local Variables:
 # mode: perl
 # perl-indent-level: 4
